//+------------------------------------------------------------------+
//|                                                    Toned_WPR.mq5 |
//|                                               Copyright TheXpert |
//|                                           theforexpert@gmail.com |
//+------------------------------------------------------------------+
#property copyright "TheXpert"
#property link      "theforexpert@gmail.com"
#property version   "1.00"
//--- connect include file
#include <Color.mqh>
//--- display the indicator in a separate subwindow
#property indicator_separate_window
//--- indicator settings
#property indicator_minimum -100 // indicator window's lower corner
#property indicator_maximum 0    // indicator window's upper corner
#property indicator_buffers 2    // number of indicator buffers
#property indicator_plots   1    // number of plots
//--- plot 1
#property indicator_type1   DRAW_COLOR_LINE
#define C clrRed // color constant
#property indicator_color1  C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C,C
#property indicator_width1  2
//--- input parameters
input int   WPRPeriod=15;   // Indicator calculation period
input int   ColorsCount=63; // Number of indicator colors
input color Lower=clrRed;   // Color's lower border
input color Higher=clrBlue; // Color's upper border
//--- indicator buffers
double Values[];
double Color[];
//--- Larry Williams' Percent Range indicator handle
int    WPRHandle;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
//--- bind indicator buffer
   SetIndexBuffer(0,Values,INDICATOR_DATA);
   SetIndexBuffer(1,Color,INDICATOR_COLOR_INDEX);
//--- set the number of initial bars without drawing
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,WPRPeriod);
//--- define the indicator color
   FillColorTable(0,Lower,Higher,ColorsCount);
//--- set the indicator short name
   IndicatorSetString(INDICATOR_SHORTNAME,"Toned WPR("+string(WPRPeriod)+")");
//--- set plot value without display
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//--- prohibit display of plot values in DataWindow
   PlotIndexSetInteger(0,PLOT_SHOW_DATA,false);
//--- receive iWPR indicator handle
   WPRHandle=iWPR(NULL,0,WPRPeriod);
//--- index elements in indicator buffers as timeseries
   ArraySetAsSeries(Values,true);
   ArraySetAsSeries(Color,true);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//--- number of bars for calculation
   int toCount=rates_total-(int)MathMax(prev_calculated-1,0);
//--- try to copy iWPR indicator data
   int copied=CopyBuffer(WPRHandle,0,0,toCount,Values);
   int err=GetLastError();
//--- check coying result
   if(copied==-1)
     {
      //--- if error number is 4806, the data simply have not been uploaded yet
      if(err==4806)
        {
         //--- wait till the data is uploaded
         for(int i=0;i<1000;++i)
            if(BarsCalculated(WPRHandle)>0)
               break;
         //--- try to copy iWPR indicator data again
         copied=CopyBuffer(WPRHandle,0,0,rates_total,Values);
         err=GetLastError();
        }
     }
//--- check coying result
   if(copied==-1)
     {
      Print("Error when trying to get WPR values, last error is ",err," bars ",rates_total);
      return(0);
     }
//--- the loop for calculating the indicator values
   for(int i=toCount-1;i>=0;--i)
     {
      Color[i]=0;
      //--- pass to the next iteration in case there is no value
      if(Values[i]==EMPTY_VALUE)
         continue;
      //--- receive color index
      int index=(int)((Values[i]+100)/100*ColorsCount);
      if(index>=ColorsCount)
         index=ColorsCount-1;
      if(index<0)
         index=0;
      //--- set color index
      Color[i]=index;
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
